unit ImgManip19125u1;

(*See const sPrgmName and sVers*)


(*===== This is...
A "skelton" which can be used as the basis for
a program to have fun manipulating images.

It is built around two Timage type controls,
imgRaw and imgDest. The idea is that you load
some image into imgRaw (the code for that is
here already) and then, by clicking buFillDest
you cause a new image to be built in imgDest.
Code to copy what's in imgRaw to imgDest is
already present in this... all you get is a
copy. Your job: Make the imgDest more interesting
than a mere copy.

To build it out past the original state, most of
the changes should go inside the
buFillDestImgClick event handler.

I have set the Stretch and Proportional properties
of both of the image controls true. This has
several worth-the-trouble advantages... but it
also introduces complications!

As supplied, the this "skeleton" works best with
images which are 146 x 120 pixels... or thereabouts.
This IS a serious limitiation, and the user is
encouraged to work at removing the limiltation.
But in early trials, you will save yourself hassles
if you keep to those dimensions. A suitable image
is supplied with the code, and the third party
graphics package "Irvanview" has a good re-sizing
tool. https://www.irfanview.com/

Play with different images as the source image,
and you will, I think, begin to see what I mean!

*)

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics,
  Dialogs, StdCtrls, ExtCtrls, IniFiles;

const sPrgmName='ImgManip19125';
      sVers='26 Jan 19';

      sIniFileName='ImgManip19125ini.txt';

type

  { TImgManip19125f1 }

  TImgManip19125f1 = class(TForm)
    buQuit: TButton;
    buLoadRawImage: TButton;
    buFillDestImg: TButton;
    buSaveDestImage: TButton;
    imgDest: TImage;
    imgRaw: TImage;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure buFillDestImgClick(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buSaveDestImageClick(Sender: TObject);
    procedure buLoadRawImageClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    bTmp:byte;
    bmDest:TBitmap;
    iImgRawWidth,iImgRawHeight:integer;
    procedure LoadImageFromFile(imgTmp: TImage; sWhatFile: String);

  public
    { public declarations }
  end;

var
  ImgManip19125f1: TImgManip19125f1;

implementation

{$R *.lfm}

{ TImgManip19125f1 }

procedure TImgManip19125f1.FormCreate(Sender: TObject);
var dfIniFile:TIniFile;//"df" in name from "datafile".
begin
  ImgManip19125f1.caption:=sPrgmName+', version '+sVers;
  application.title:=sPrgmName;

//Next: use an ini file to re-open window in
//   previous state...
//See http://sheepdogguides.com/lut/lt2Nc.htm
//   for a tutorial on ini files. Be sure to
//   put IniFiles in the unit's uses if you
//   want to use ini files.

dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+sIniFileName);

with dfIniFile do begin
   ImgManip19125f1.height:=ReadInteger('WindowProperties','Height',340);
   ImgManip19125f1.width:=ReadInteger('WindowProperties','Width',480);
   ImgManip19125f1.top:=ReadInteger('WindowProperties','Top',10);
   ImgManip19125f1.left:=ReadInteger('WindowProperties','Left',10);
   end;//with...

dfIniFile.Free;
//End "... using an ini file..."

imgRaw.Stretch:=true;
imgRaw.Proportional:=true;//Image seems to clip to be as big as
//poss without losing aspect ratio inside area allocated for
//the img object. Both height and width respected, neither allowed
//to be BIGGER than allocation, one is reduced.
imgDest.Stretch:=true;
imgDest.Proportional:=true;

iImgRawHeight:=imgRaw.height;
iImgRawWidth:=imgRaw.width;

imgDest.height:=iImgRawHeight;
imgDest.width:=iImgRawWidth;

bmDest:=TBitmap.create;//Create a bitmap object
bmDest.height:=iImgRawHeight;//Assign dimensions
bmDest.width:=iImgRawWidth;

//In Delphi, bitmap background is white to begin with. Black in Lazarus
//bmDest.canvas.pen.color:=clWhite;//1 of 2 lines not needed in Delphi...
//bmDest.canvas.Rectangle(0,0,imgDest.width,imgDest.height);//2 of 2 not needed, Delphi

imgDest.Picture.Graphic:=bmDest; //Assign the bitmap to the image component

imgDest.visible:=false;//To tidy a minor bit of unsightliness! Rem
//out, run program. Look at result, before a FillDestImg has been
//done, to see why this visible:=false put in.

end;//FormCreate

procedure TImgManip19125f1.FormClose(Sender: TObject; var CloseAction: TCloseAction
  );
var dfIniFile:TInifile;
begin
//Next uses an ini file to save window's
//    properties, so it can open next time
//    as it is now...

dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+sIniFileName);
 with dfIniFile do begin

  WriteInteger('WindowProperties','Height',ImgManip19125f1.height);
  WriteInteger('WindowProperties','Width',ImgManip19125f1.width);
  WriteInteger('WindowProperties','Top',ImgManip19125f1.top);
  WriteInteger('WindowProperties','Left',ImgManip19125f1.left);

  end;//with...
dfIniFile.Free;
//end of saving data to ini file.
end;//FormClose

procedure TImgManip19125f1.buQuitClick(Sender: TObject);
begin
  close;//application.terminate; does NEARLY the same thing,
  //but fails to call the OnFormClose handler... a problem
  //for the ini file system built into this.
end;

procedure TImgManip19125f1.buSaveDestImageClick(Sender: TObject);
begin
  //This code needs to be written. It should make a copy of
  //the destination image on backing store, when this button
  //is pressed.
end;


procedure TImgManip19125f1.LoadImageFromFile(imgTmp: TImage; sWhatFile: String);
//Modified by TKB... was initially to load image from a data stream.
////THANKS TO...
//https://stackoverflow.com/questions/10200252/jpeg-data-stream-to-timage
//(TLama) for the basis of this subroutine.

var
  JPEGImage: TJPEGImage;
begin
  //AData.Position := 0;
  JPEGImage := TJPEGImage.Create;
  try
    JPEGImage.LoadFromFile(sWhatFile);
    imgTmp.Picture.Assign(JPEGImage);
  finally
    JPEGImage.Free;
  end;
end;

procedure TImgManip19125f1.buLoadRawImageClick(Sender: TObject);
var sFilename:String;
begin
//hardcoded image size... very limiting!.. and not 100% successful... yet!
  imgRaw.height:=120;
  imgRaw.width:=146;
  sFilename:='Kids-146x120Train-18411-Centre-18411-17-00C8037.jpg';
  LoadImageFromFile(imgRaw,sFilename);

  imgDest.Picture.Bitmap.height:=imgRaw.height;
  imgDest.Picture.Bitmap.width:=imgRaw.width;

  imgDest.height:=imgRaw.height;
  imgDest.width:=imgRaw.width;

end;//buLoadRawImageClick

procedure TImgManip19125f1.buFillDestImgClick(Sender: TObject);
var iX, iY:integer;
    clPixel:TColor;
    bCompRed,bCompBlue,bCompGreen:byte;//for R,G,B
        //components of a color

//This is the heart of this program as a "shell".
//At the moment, the "raw" image is merely copied,
//pixel by pixel to the "destination".
//
//The fun will be in "doing things" between picking
//up a pixel (or group of pixels!) from the raw
//image, and Doing Things to them before "painting"
//to the destionation.
//
//(A simple exercise: Make the destination a mirror
//image of the source.)

begin
imgDest.visible:=true;//No rocket science behind
 //this.. the empty image control was unsightly, that's
 //all.
for iY:=1 to imgRaw.height do begin
  for iX:=1 to imgRaw.width do begin
    clPixel:=imgRaw.Picture.Bitmap.canvas.Pixels[iX,iY];

    //The following is a start at ONE possible transformation...
    bCompRed:=Red(clPixel);
    bCompGreen:=Green(clPixel);
    bCompBlue:=Blue(clPixel);

    (* An idea for a different transform... replace the
     "swap colors" code, if trying this...
    if bCompRed>190 then bCompBlue:=0;
    if bCompBlue>190 then bCompGreen:=0;
    *)

    //Next 4 constitute "swap colors" code...
    bTmp:=bCompRed;
    bCompRed:=bCompGreen;
    bCompGreen:=bCompBlue;
    bCompBlue:=bTmp;

    (*following didn't work... why not??
    bCompRed:=(bCompRed mod 4)*64;
    bCompBlue:=(bCompBlue mod 4)*64;
    bCompGreen:=(bCompGreen mod 4)*64;  *)

    clPixel:=RGBToColor(bCompRed,bCompGreen,bCompBlue);

    //... and now write result to imgDest
    imgDest.Picture.Bitmap.canvas.Pixels[iX,iY]:=clPixel;

  end;//For iY
end;//For iY
end;//buFilDestImgClick

end.
